@@ -11,7 +11,8 @@ use serde::{Deserialize, Serialize};
 pub struct CsmlBot {
     pub id: String,
     pub name: String,
-    pub fn_endpoint: Option<String>,
+    #[serde(alias = "fn_endpoint")]
+    pub apps_endpoint: Option<String>,
     pub flows: Vec<CsmlFlow>,
     pub native_components: Option<serde_json::Map<String, serde_json::Value>>, //
     pub custom_components: Option<serde_json::Value>,                          // serde_json::Value
@@ -29,7 +30,7 @@ impl CsmlBot {
     pub fn new(
         id: &str,
         name: &str,
-        fn_endpoint: Option<String>,
+        apps_endpoint: Option<String>,
         flows: Vec<CsmlFlow>,
         native_components: Option<serde_json::Map<String, serde_json::Value>>,
         custom_components: Option<serde_json::Value>,
@@ -41,7 +42,7 @@ impl CsmlBot {
         Self {
             id: id.to_owned(),
             name: name.to_owned(),
-            fn_endpoint,
+            apps_endpoint,
             flows,
             native_components,
             custom_components,
@@ -70,4 +71,18 @@ impl CsmlBot {
             format!("{} {}", ERROR_INVALID_FLOW, name),
         )])
     }
+
+    pub fn to_json(&self) -> serde_json::Value {
+        let mut map: serde_json::Map<String, serde_json::Value> = serde_json::Map::new();
+
+        map.insert("id".to_owned(), serde_json::json!(self.id));
+        map.insert("name".to_owned(), serde_json::json!(self.name));
+        map.insert("apps_endpoint".to_owned(), serde_json::json!(self.apps_endpoint));
+        map.insert("flows".to_owned(), serde_json::json!(self.flows));
+        map.insert("default_flow".to_owned(), serde_json::json!(self.default_flow));
+        map.insert("no_interruption_delay".to_owned(), serde_json::json!(self.no_interruption_delay));
+        map.insert("env".to_owned(), serde_json::json!(self.env));
+
+        serde_json::json!(map)
+    }
 }
